package com.zanxiang.manage.service.Impl;

import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.core.toolkit.CollectionUtils;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.zanxiang.common.utils.StringUtils;
import com.zanxiang.common.utils.bean.BeanUtils;
import com.zanxiang.manage.domain.dto.UserDTO;
import com.zanxiang.manage.domain.dto.UserExtDTO;
import com.zanxiang.manage.domain.params.UserNameAuthListParam;
import com.zanxiang.manage.domain.vo.UserNameAuthVO;
import com.zanxiang.manage.service.UserExtService;
import com.zanxiang.manage.service.UserService;
import com.zanxiang.mybatis.entity.UserExt;
import com.zanxiang.mybatis.mapper.UserExtMapper;
import lombok.extern.slf4j.Slf4j;
import org.apache.logging.log4j.util.Strings;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import java.time.LocalDateTime;
import java.time.LocalTime;
import java.util.Collections;
import java.util.List;
import java.util.Map;
import java.util.Objects;
import java.util.function.Function;
import java.util.stream.Collectors;

/**
 * @author : lingfeng
 * @time : 2022-06-29
 * @description : 玩家拓展信息
 */
@Slf4j
@Service
public class UserExtServiceImpl extends ServiceImpl<UserExtMapper, UserExt> implements UserExtService {

    @Autowired
    private UserService userService;

    /**
     * 多条件查询用户拓展信息
     *
     * @param realName : 真实姓名
     * @param regIp    : 注册ip
     * @param isAuth   : 是否实名
     * @return : 返回用户拓展信息列表
     */
    @Override
    public Map<Long, UserExtDTO> getUserExtList(String realName, String regIp, Boolean isAuth) {
        if (Strings.isBlank(realName) && Strings.isBlank(regIp) && isAuth == null) {
            return null;
        }
        List<UserExt> userExtList = super.list(new LambdaQueryWrapper<UserExt>()
                .eq(Strings.isNotBlank(realName), UserExt::getRealName, realName)
                .eq(Strings.isNotBlank(regIp), UserExt::getRegIp, regIp)
                .eq(isAuth != null, UserExt::getIsAuth, isAuth)
                .select(UserExt::getUserId, UserExt::getRealName, UserExt::getRegIp, UserExt::getIsAuth, UserExt::getIdCard));
        if (CollectionUtils.isEmpty(userExtList)) {
            return Collections.emptyMap();
        }
        return BeanUtils.copyList(userExtList, UserExtDTO.class).stream()
                .collect(Collectors.toMap(UserExtDTO::getUserId, Function.identity()));
    }

    /**
     * 注册ip相关条件查询用户拓展信息
     *
     * @param regIp : 参数
     * @return : 玩家信息
     */
    @Override
    public Map<Long, UserExtDTO> regIpCondition(String regIp) {
        if (Strings.isBlank(regIp)) {
            return null;
        }
        //玩家注册ip条件处理
        List<UserExt> userExtList = super.list(new LambdaQueryWrapper<UserExt>()
                .eq(UserExt::getRegIp, regIp)
                .select(UserExt::getUserId, UserExt::getRegIp));
        if (CollectionUtils.isEmpty(userExtList)) {
            return Collections.emptyMap();
        }
        return BeanUtils.copyList(userExtList, UserExtDTO.class).stream()
                .collect(Collectors.toMap(UserExtDTO::getUserId, Function.identity()));
    }

    /**
     * 根据玩家id查询
     *
     * @param userId : 玩家id
     * @return : 返回玩家信息
     */
    @Override
    public UserExtDTO getById(Long userId) {
        if (userId == null) {
            return null;
        }
        return BeanUtils.copy(super.getById(userId), UserExtDTO.class);
    }

    /**
     * 获取玩家实名认证列表
     *
     * @param param : 查询参数
     * @return : 返回单页数据
     */
    @Override
    public IPage<UserNameAuthVO> authList(UserNameAuthListParam param) {
        Long userId = param.getUserId();
        String userName = param.getUserName();
        String nickname = param.getNickname();
        //玩家账号和玩家昵称涉及的玩家条件
        List<UserDTO> userDTOList = userService.getUserList(userId, null, userName, nickname);
        //玩家信息map
        Map<Long, UserDTO> map = userDTOList.stream().collect(Collectors.toMap(UserDTO::getId, Function.identity()));
        //玩家id列表
        List<Long> userIdList = userDTOList.stream().map(UserDTO::getId).collect(Collectors.toList());
        //查询返回
        return page(param.toPage(), new QueryWrapper<UserExt>().lambda()
                .eq(UserExt::getIsAuth, Boolean.TRUE)
                .eq(userId != null, UserExt::getUserId, userId)
                .in(CollectionUtils.isNotEmpty(userIdList), UserExt::getUserId, userIdList)
                .ge(param.getBeginDate() != null, UserExt::getCreateTime, param.getBeginDate() == null ? null : LocalDateTime.of(param.getBeginDate(), LocalTime.MIN))
                .le(param.getEndDate() != null, UserExt::getCreateTime, param.getEndDate() == null ? null : LocalDateTime.of(param.getEndDate(), LocalTime.MAX))
                .eq(param.getPlatformId() != null, UserExt::getAuthPlatform, param.getPlatformId())
                .eq(Strings.isNotBlank(param.getRealName()), UserExt::getRealName, param.getRealName())
                .eq(Strings.isNotBlank(param.getIdCard()), UserExt::getIdCard, param.getIdCard())
                .eq(Strings.isNotBlank(param.getZxbPi()), UserExt::getZxbPi, param.getZxbPi())
                .orderByDesc(UserExt::getCreateTime))
                .convert(u -> this.toVo(u, map));
    }

    /**
     * 玩家实名认证信息转化
     *
     * @param userExt : 玩家实名信息
     * @return : 返回玩家实名信息
     */
    private UserNameAuthVO toVo(UserExt userExt, Map<Long, UserDTO> map) {
        UserNameAuthVO userNameAuthVO = BeanUtils.copy(userExt, UserNameAuthVO.class);
        if (Objects.isNull(userNameAuthVO)) {
            return null;
        }
        UserDTO userDTO = StringUtils.isEmpty(map) ? null : map.get(userNameAuthVO.getUserId());
        if (userDTO == null) {
            userDTO = userService.getById(userNameAuthVO.getUserId());
        }
        if (userDTO != null) {
            userNameAuthVO.setUsername(userDTO.getUsername());
            userNameAuthVO.setNickname(userDTO.getNickname());
        }
        return userNameAuthVO;
    }


}
